#include <iostream>
#include "ball.h"



Ball::Ball(	float centerX,
		float centerY,
		float centerZ,
		float centerVX,
		float centerVY,
		float centerVZ,
		int numMassPoints,
		float massPoints[][3],
		int numTriangles,
		int triangles[][3]) {
    
    numOfMassPoints = numMassPoints;
    massPoints_ = new MassPoint[numOfMassPoints+1]; // +1 for the center
    radiusOfBoundingSphere = 0.0;

    // center:
    massPoints_[numOfMassPoints].position[0] = centerX;
    massPoints_[numOfMassPoints].position[1] = centerY;
    massPoints_[numOfMassPoints].position[2] = centerZ;
    massPoints_[numOfMassPoints].velocity[0] = centerVX;
    massPoints_[numOfMassPoints].velocity[1] = centerVY;
    massPoints_[numOfMassPoints].velocity[2] = centerVZ;
    massPoints_[numOfMassPoints].force[0] = 0.0;
    massPoints_[numOfMassPoints].force[1] = 0.0;
    massPoints_[numOfMassPoints].force[2] = 0.0;
    massPoints_[numOfMassPoints].penaltyForce[0] = 0.0;
    massPoints_[numOfMassPoints].penaltyForce[1] = 0.0;
    massPoints_[numOfMassPoints].penaltyForce[2] = 0.0;
    massPoints_[numOfMassPoints].mass = 1.0; // 1.0;
    
    for(int i=0; i< numOfMassPoints; i++) {
	massPoints_[i].position[0] = centerX + massPoints[i][0];
	massPoints_[i].position[1] = centerY + massPoints[i][1];
	massPoints_[i].position[2] = centerZ + massPoints[i][2];
	massPoints_[i].velocity[0] = centerVX;
	massPoints_[i].velocity[1] = centerVY;
	massPoints_[i].velocity[2] = centerVZ;
	massPoints_[i].force[0] = 0.0;
	massPoints_[i].force[1] = 0.0;
	massPoints_[i].force[2] = 0.0;
	massPoints_[i].penaltyForce[0] = 0.0;
	massPoints_[i].penaltyForce[1] = 0.0;
	massPoints_[i].penaltyForce[2] = 0.0;
	massPoints_[i].mass = 1.0; // 1.0
	if (distance(i, numOfMassPoints) > radiusOfBoundingSphere) {
	  radiusOfBoundingSphere = distance(i, numOfMassPoints);
	}
    }
    
    numOfTriangles = numTriangles;
    triangles_ = new int[numOfTriangles][3];
    
	
    for(int i=0; i< numOfTriangles; i++) {
	triangles_[i][0] = triangles[i][0];
	triangles_[i][1] = triangles[i][1];
	triangles_[i][2] = triangles[i][2];
    }

    numOfSprings = ((3*numTriangles)/2) + numMassPoints;
//    std::cout << "numOfSprings: " << numOfSprings << std::endl;
    springs = new Spring[numOfSprings];

    
    //creating springs:

    bool pending[numOfMassPoints];
    int springcounter = 0;
    for(int i = 0; i<numOfMassPoints; ++i) {
	for(int k=0; k < numOfMassPoints; ++k)
	    pending[k] = true;
	for(int j = 0; j < numOfTriangles; ++j) {
	    if (triangles_[j][0] == i) {
		if ((triangles_[j][1] > i) && (pending[triangles_[j][1]])) {
		    springs[springcounter].massPoint1 = i;
		    springs[springcounter].massPoint2 = triangles_[j][1];
		    pending[triangles_[j][1]] = false;
		    springs[springcounter].springConstant = PERIPHERAL_SPRING_CONSTANT;
		    springs[springcounter].initLength = distance(i, triangles_[j][1]);
		    springs[springcounter].lastLength = distance(i, triangles_[j][1]);
		    ++springcounter;
		}
		if ((triangles_[j][2] > i) && (pending[triangles_[j][2]])) {
		    springs[springcounter].massPoint1 = i;
		    springs[springcounter].massPoint2 = triangles_[j][2];
		    pending[triangles_[j][2]] = false;
		    springs[springcounter].springConstant = PERIPHERAL_SPRING_CONSTANT;
		    springs[springcounter].initLength = distance(i, triangles_[j][2]);
		    springs[springcounter].lastLength = distance(i, triangles_[j][2]);
		    ++springcounter;
		}
	    }
	    else if (triangles_[j][1] == i) {
		if ((triangles_[j][0] > i) && (pending[triangles_[j][0]])) {
		    springs[springcounter].massPoint1 = i;
		    springs[springcounter].massPoint2 = triangles_[j][0];
		    pending[triangles_[j][0]] = false;
		    springs[springcounter].springConstant = PERIPHERAL_SPRING_CONSTANT;
		    springs[springcounter].initLength = distance(i, triangles_[j][0]);
		    springs[springcounter].lastLength = distance(i, triangles_[j][0]);
		    ++springcounter;
		}
		if ((triangles_[j][2] > i) && (pending[triangles_[j][2]])) {
		    springs[springcounter].massPoint1 = i;
		    springs[springcounter].massPoint2 = triangles_[j][2];
		    pending[triangles_[j][2]] = false;
		    springs[springcounter].springConstant = PERIPHERAL_SPRING_CONSTANT;
		    springs[springcounter].initLength = distance(i, triangles_[j][2]);
		    springs[springcounter].lastLength = distance(i, triangles_[j][2]);
		    ++springcounter;
		}
	    }
	    else if (triangles_[j][2] == i) {
		if ((triangles_[j][0] > i) && (pending[triangles_[j][0]])) {
		    springs[springcounter].massPoint1 = i;
		    springs[springcounter].massPoint2 = triangles_[j][0];
		    pending[triangles_[j][0]] = false;
		    springs[springcounter].springConstant = PERIPHERAL_SPRING_CONSTANT;
		    springs[springcounter].initLength = distance(i, triangles_[j][0]);
		    springs[springcounter].lastLength = distance(i, triangles_[j][0]);
		    ++springcounter;
		}
		if ((triangles_[j][1] > i) && (pending[triangles_[j][1]])) {
		    springs[springcounter].massPoint1 = i;
		    springs[springcounter].massPoint2 = triangles_[j][1];
		    pending[triangles_[j][1]] = false;
		    springs[springcounter].springConstant = PERIPHERAL_SPRING_CONSTANT;
		    springs[springcounter].initLength = distance(i, triangles_[j][1]);
		    springs[springcounter].lastLength = distance(i, triangles_[j][1]);
		    ++springcounter;
		}
	    }
	    
	}

	springs[springcounter].massPoint1 = i;
	springs[springcounter].massPoint2 = numOfMassPoints;
	springs[springcounter].springConstant = RADIAL_SPRING_CONSTANT;
	springs[springcounter].initLength = distance(i, numOfMassPoints);
	springs[springcounter].lastLength = distance(i, numOfMassPoints);
	++springcounter;
	
    }

	
    
    
    
	

/*    

      // spring creation for rhombus (TEST)

    springs[0].massPoint1 = 0;
    springs[0].massPoint2 = 2;
    springs[1].massPoint1 = 2;
    springs[1].massPoint2 = 4;
    springs[2].massPoint1 = 2;
    springs[2].massPoint2 = 1;
    springs[3].massPoint1 = 1;
    springs[3].massPoint2 = 4;
    springs[4].massPoint1 = 1;
    springs[4].massPoint2 = 3;
    springs[5].massPoint1 = 4;
    springs[5].massPoint2 = 3;
    springs[6].massPoint1 = 0;
    springs[6].massPoint2 = 3;
    springs[7].massPoint1 = 0;
    springs[7].massPoint2 = 4;
    springs[8].massPoint1 = 0;
    springs[8].massPoint2 = 5;
    springs[9].massPoint1 = 2;
    springs[9].massPoint2 = 5;
    springs[10].massPoint1 = 1;
    springs[10].massPoint2 = 5;
    springs[11].massPoint1 = 3;
    springs[11].massPoint2 = 5;
    
    for (int sp=0; sp < numOfSprings; sp++) {
    	if (sp < numOfSprings-numMassPoints) {
    	  // spring on the surface:
    	  springs[sp].initLength = sqrt(2);
    	  springs[sp].springConstant = 0.5; // stiffness k
    	  
    	  // TODO (Koni): define massPoint1 and massPoint2
    	  
    	} else {
    	  // spring to the center:
    	  springs[sp].initLength = 1.3;  // TESTING: set to 1.3 instead of 1.0 to see if it expannds
    	  springs[sp].springConstant = 0.5; // stiffness k
    	  springs[sp].massPoint1 = sp - ((3*numTriangles)/2);
    	  springs[sp].massPoint2 = numOfMassPoints;
        }
    }
*/
    
    vertexNormals_ = new float[numOfMassPoints][3];

    faceNormals_ = new float[numOfTriangles][3];
    
    //set rendering issues to default values:
    ambient[0] = 0.2; ambient[1] = 0.0; ambient[2] = 0.0; ambient[3] = 1.0;
    diffuse[0] = 0.8; diffuse[1] = 0.0; diffuse[2] = 0.0; diffuse[3] = 1.0; 
    specular[0] = 0.0; specular[1] = 0.0; specular[2] = 0.0; specular[3] = 1.0; 
    shininess = 65.0;
}


Ball::~Ball() {
//    delete[] meshPoints_;
    delete[] triangles_;
    delete[] vertexNormals_;
    delete[] faceNormals_;
}

void Ball::rotate(float axisX, float axisY, float axisZ, float phi) {
}

void Ball::translate(float dirX, float dirY, float dirZ) {
}

void Ball::draw() {
    int indexA, indexB, indexC;

    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);
    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, shininess);

    calculateVertexNormals();

    
    for(int triangle_index = 0; triangle_index < numOfTriangles; triangle_index++) {
    
	indexA = triangles_[triangle_index][0];
	indexB = triangles_[triangle_index][1];
	indexC = triangles_[triangle_index][2];
    
	glBegin(GL_TRIANGLES);

	// with face normals (for FLAT shading)
	// glNormal3fv(faceNormals_[triangle_index]);
	    
	glNormal3fv(vertexNormals_[indexA]);
	glVertex3fv(massPoints_[indexA].position);
    
	glNormal3fv(vertexNormals_[indexB]);
	glVertex3fv(massPoints_[indexB].position);
    
	glNormal3fv(vertexNormals_[indexC]);
	glVertex3fv(massPoints_[indexC].position);
    
	glEnd();
    }



}

void Ball::calculateVertexNormals() {

    float vertexA[3];
    float vertexB[3];
    float vertexC[3];

    int vertexA_index;
    int vertexB_index;
    int vertexC_index;

    int nbr_of_normals[numOfMassPoints];

    float normal[3];
    float normalLength;

    float vectorAB[3];
    float vectorAC[3];

    //clear nbr_of_normals Array:
    for(int i = 0; i < numOfMassPoints; ++i)
	nbr_of_normals[i] = 0;
    

    for(int triangle_index = 0; triangle_index < numOfTriangles; ++triangle_index) {

	vertexA_index = triangles_[triangle_index][0];
	vertexB_index = triangles_[triangle_index][1];
	vertexC_index = triangles_[triangle_index][2];

	
	for(int i=0; i < 3; i++) {
	    vertexA[i] = massPoints_[vertexA_index].position[i];
	    vertexB[i] = massPoints_[vertexB_index].position[i];
	    vertexC[i] = massPoints_[vertexC_index].position[i];
	}

	for(int i=0; i < 3; i++) {
	    vectorAB[i] = vertexB[i] - vertexA[i];
	    vectorAC[i] = vertexC[i] - vertexA[i];
	}

	normal[0] = vectorAB[1]*vectorAC[2] - vectorAB[2]*vectorAC[1];
	normal[1] = vectorAB[2]*vectorAC[0] - vectorAB[0]*vectorAC[2];
	normal[2] = vectorAB[0]*vectorAC[1] - vectorAB[1]*vectorAC[0];
	normalLength = sqrt(normal[0]*normal[0]+normal[1]*normal[1]+normal[2]*normal[2]);
	
	normal[0] = normal[0] / normalLength;
	normal[1] = normal[1] / normalLength;
	normal[2] = normal[2] / normalLength;

	
	for(int i=0; i < 3; i++) {
	    
	    faceNormals_[triangle_index][i] = normal[i];
	    vertexNormals_[vertexA_index][i] += normal[i];
	    vertexNormals_[vertexB_index][i] += normal[i];
	    vertexNormals_[vertexC_index][i] += normal[i];
	}
	
	++nbr_of_normals[vertexA_index];
	++nbr_of_normals[vertexB_index];
	++nbr_of_normals[vertexC_index];

    }

    for(int i = 0; i < numOfMassPoints; ++i) {
	vertexNormals_[i][0] /= nbr_of_normals[i];
	vertexNormals_[i][1] /= nbr_of_normals[i];
	vertexNormals_[i][2] /= nbr_of_normals[i];
    }

}

float Ball::distance(int pointA, int pointB) {
    
    float x_dist, y_dist, z_dist;
    
    x_dist = massPoints_[pointB].position[0] - massPoints_[pointA].position[0];
    y_dist = massPoints_[pointB].position[1] - massPoints_[pointA].position[1];
    z_dist = massPoints_[pointB].position[2] - massPoints_[pointA].position[2];

    return sqrt(x_dist*x_dist + y_dist*y_dist + z_dist*z_dist);
}

    
