#ifndef _BALL_H
#define _BALL_H

#include <math.h>
#include <GL/glut.h>


#define PERIPHERAL_SPRING_CONSTANT 3.5 // 1.2
#define RADIAL_SPRING_CONSTANT 3.5 // 1.2


typedef struct spring_desc {
    float initLength;	// L
    float springConstant; // stiffness k
    //index of first attached masspoint
    int massPoint1;
    //index of second attached masspoint
    int massPoint2;
    float lastLength;
} Spring;

typedef struct massPoint_desc {
    float mass;
    float velocity[3];  // the current speed vector
    float force[3];  // for the calculation of the force
    float penaltyForce[3];
    float position[3];
} MassPoint;




class Ball {

    private:
	
	int (*triangles_)[3];
	int numOfTriangles;

	// not necessary?? we compute vertex normals directly
	// for flat shading??
	float (*faceNormals_)[3];
	
	float (*vertexNormals_)[3];
	
	//mass of a peripheral masspoint
	float peripheralMass;
	//mass of the central masspoint
	float centralMass;
	//spring constant of a peripheral spring
	float peripheralSpringConstant;
	//spring constant of a radial spring
	float radialSpringConstant;
	void calculateVertexNormals();



	//rendering issues
	GLfloat ambient[4];
	GLfloat diffuse[4];
	GLfloat specular[4];
	GLfloat shininess;
	

    protected:

    public:
	
	// center of the ball
	float center_[3];

	// array that holds control points of the triangle mesh
	MassPoint (*massPoints_);
	int numOfMassPoints;
	
	// Radius of the bounding sphere centered at the center point of the ball
	// This is used for collision detection between two balls.
	float radiusOfBoundingSphere;

	//calculates distance between points with index pointA and pointB
	float distance(int pointA, int pointB);

	//array that holds all springs of the ball
	Spring (*springs);
	int numOfSprings;  // the number of springs

	Ball(	float centerX,
		float centerY,
		float centerZ,
		float centerVX,
		float centerVY,
		float centerVZ,
		int numMassPoints,
		float massPoints[][3],
		int numTriangles,
		int triangles[][3]);

	~Ball();

	void rotate(float axisX, float axisY, float axisZ, float phi);
	void translate(float dirX, float dirY, float dirZ);
	void draw();


};

#endif


