#include <iostream>
#include "cube.h"


Cube::Cube(float centerX, float centerY, float centerZ, float edgeLength) {

    center_[0] = centerX;
    center_[1] = centerY;
    center_[2] = centerZ;
    edgeLength_ = edgeLength;
    xLower_ = center_[0] - edgeLength/2.0;
    xUpper_ = center_[0] + edgeLength/2.0;
    yLower_ = center_[1] - edgeLength/2.0;
    yUpper_ = center_[1] + edgeLength/2.0;
    zLower_ = center_[2] - edgeLength/2.0;
    zUpper_ = center_[2] + edgeLength/2.0;


    // Load the texture
    readTextureFile();
}

Cube::~Cube() {
}


//rotation 
void Cube::rotate(float alpha, float beta){
}

//translation
void Cube::translate(float dirX, float dirY, float dirZ){
}

//ckeck if a ball is completly inside the cube
//
//

//check if a point is inside the cube
bool Cube::inside(float pointX, float pointY, float pointZ, float *collisionPointX, float *collisionPointY, float *collisionPointZ){

    if (   (pointX > xUpper_) || (pointX < xLower_)
	|| (pointY > yUpper_) || (pointY < yLower_)
	|| (pointZ > zUpper_) || (pointZ < zLower_)) {
	

	float minOut = 0.0;
	
	if ((pointX - xUpper_) > minOut) {
	    minOut = pointX - xUpper_;
	    *collisionPointX = xUpper_;
	}
	else if ((-pointX + xLower_) > minOut) {
	    minOut = -pointX + xLower_;
	    *collisionPointX = xLower_;
	}
	else
	    *collisionPointX = pointX;
	    
	if ((pointY - yUpper_) > minOut) {
	    minOut = pointY - yUpper_;
	    *collisionPointX = pointX;
	    *collisionPointY = yUpper_;
	}
	else if ((-pointY + yLower_) > minOut) {
	    minOut = -pointY - yLower_;
	    *collisionPointX = pointX;
	    *collisionPointY = yLower_;
	}
	else
	    *collisionPointY = pointY;

	if ((pointZ - zUpper_) > minOut) {
	    *collisionPointX = pointX;
	    *collisionPointY = pointY;
	    *collisionPointZ = zUpper_;
	}
	else if ((-pointZ + zLower_) > minOut) {
	    *collisionPointX = pointX;
	    *collisionPointY = pointY;
	    *collisionPointZ = zLower_;
	}
	else
	    *collisionPointZ = pointZ;

	return false;
    }
    
    return true;
    
}

bool Cube::readTexmap(char *acFile, GLubyte TexMap[TEXTURE_SIZE][TEXTURE_SIZE][3])
{
  FILE* in;
  int i, j, xsize, ysize;
  unsigned char ch, buf[200];
  
  in = fopen(acFile, "rb");
  if( !in ){
    printf("ERROR: dwsa_read_texmap: bad fileopen\n");
    glDisable(GL_TEXTURE_2D);
    return false;
  }
  
  /*read header*/
  fread(buf, 2, 1, in); /*magic num*/
  if( (buf[0] != 'P') || (buf[1] != '6') ){
    printf("ERROR: dwsa_read_texmap: bad file magicnum\n");
    glDisable(GL_TEXTURE_2D);
    return false;
  }
  /*skip whitespace*/
  fread(&buf[0], 1, 1, in);
  do
  {
    ch = buf[0];
    while( ch != '\n' ) fread(&ch, 1, 1, in);
	  fread(&buf[0], 1, 1, in);
  }while( (buf[0] < '0') || (buf[0] > '9') );
  /*get width*/
  fread(&ch, 1, 1, in);
  for(i = 1; (ch >= '0') && (ch <= '9'); i++)
  {
    buf[i] = ch;
    fread(&ch, 1, 1, in);
  }
  buf[i] = '\0';
  xsize = atoi((char *)buf);
  /*the space has been read, can continue straight*/
  fread(&ch, 1, 1, in);
  for(i = 0; (ch >= '0') && (ch <= '9'); i++)
  {
    buf[i] = ch;
    fread(&ch, 1, 1, in);
  }
  buf[i] = '\0';
  ysize = atoi((char *)buf);
  /*the newline has been read, can continue straight*/
  fread(&ch, 1, 1, in);
  for(i = 0; (ch >= '0') && (ch <= '9'); i++)
  {
    buf[i] = ch;
    fread(&ch, 1, 1, in);
  }
  buf[i] = '\0';
  
  if( (xsize != TEXTURE_SIZE) || (ysize != TEXTURE_SIZE) )
  {
    printf("ERROR: dwsa_read_texmap: bad file dims\n");
    return false;
  }
   
  /*the newline is read, continue straight*/
  for(i = 0; i < xsize; i++)
  {
	for(j = 0; j < ysize; j++)
	{
	  fread(&ch, 1, 1, in);
	  TexMap[xsize-1-i][j][0] = ch;
	  fread(&ch, 1, 1, in);
	  TexMap[xsize-1-i][j][1] = ch;
	  fread(&ch, 1, 1, in);
	  TexMap[xsize-1-i][j][2] = ch;
	}
  }
  fclose(in);
  return true;
}


void Cube::readTextureFile()
{
  GLubyte textureArray[TEXTURE_SIZE][TEXTURE_SIZE][3]; /* array for gl texture definition */

  /* read the texture with the filename texture_file_name and give back a pointer to the first element (textureArray) */
  if(readTexmap("wood.ppm", textureArray)) {
    glGenTextures(1, &woodtexture);
    /* define this texture  as current OpenGL texture */
    glBindTexture(GL_TEXTURE_2D, woodtexture);
    glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
    glTexImage2D(GL_TEXTURE_2D, 0, 3, TEXTURE_SIZE, TEXTURE_SIZE, 0,
    			 GL_RGB, GL_UNSIGNED_BYTE, textureArray);
    // Specify filtering and edge actions
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
  } else {
    std::cout << "error: could not read texture file" << std::endl;
  }
}



void Cube::draw() {
    //rendering issues
    GLfloat ambient[4];
    GLfloat diffuse[4];
    GLfloat specular[4];
    GLfloat shininess;
    GLfloat durchsichtig[4];
    
    
    ambient[0] = 1.0; ambient[1] = 1.0; ambient[2] = 1.0; ambient[3] = 1.0;
    diffuse[0] = 1.0; diffuse[1] = 1.0; diffuse[2] = 1.0; diffuse[3] = 1.0; 
    specular[0] = 1.0; specular[1] = 1.0; specular[2] = 1.0; specular[3] = 1.0; 
    shininess = 80.0;
    durchsichtig[0] = 0.0; durchsichtig[1] = 0.0; durchsichtig[1] = 0.0; durchsichtig[3] = 0.0; 
    glEnable(GL_CULL_FACE);  // damit durchsichtige Flaechen keine anderen Flaechen verdecken
    glEnable(GL_DEPTH_TEST);
    glDepthMask(GL_TRUE);
    glEnable(GL_BLEND);
    glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
    glMaterialfv(GL_FRONT, GL_AMBIENT, ambient);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, diffuse);
    glMaterialfv(GL_FRONT, GL_SPECULAR, specular);
    glMaterialf(GL_FRONT, GL_SHININESS, shininess);

    glMaterialfv(GL_BACK, GL_AMBIENT, durchsichtig);
    glMaterialfv(GL_BACK, GL_DIFFUSE, durchsichtig);
    glMaterialfv(GL_BACK, GL_SPECULAR, durchsichtig);
    glMaterialf(GL_BACK, GL_SHININESS, 0.0);

    glEnable(GL_TEXTURE_2D); // enable texturing
    glBindTexture(GL_TEXTURE_2D, woodtexture);
    
    // xUpper
    glNormal3f(-1.0, 0.0, 0.0);
    
    glBegin(GL_QUADS);
      glTexCoord2f(0.0, 0.0); glVertex3f(xUpper_, yLower_, zLower_); 
      glTexCoord2f(0.0, 1.0); glVertex3f(xUpper_, yLower_, zUpper_);
      glTexCoord2f(1.0, 1.0); glVertex3f(xUpper_, yUpper_, zUpper_);
      glTexCoord2f(1.0, 0.0); glVertex3f(xUpper_, yUpper_, zLower_);
    glEnd();

    // xLower
    glNormal3f(1.0, 0.0, 0.0);
    
    glBegin(GL_QUADS);
      glTexCoord2f(0.0, 1.0); glVertex3f(xLower_, yLower_, zLower_);
      glTexCoord2f(1.0, 1.0); glVertex3f(xLower_, yUpper_, zLower_);
      glTexCoord2f(1.0, 0.0); glVertex3f(xLower_, yUpper_, zUpper_);
      glTexCoord2f(0.0, 0.0); glVertex3f(xLower_, yLower_, zUpper_);
    glEnd();
   
    // yUpper
    glNormal3f(0.0, -1.0, 0.0);
    
    glBegin(GL_QUADS);
      glTexCoord2f(1.0, 1.0); glVertex3f(xUpper_, yUpper_, zUpper_);
      glTexCoord2f(0.0, 1.0); glVertex3f(xLower_, yUpper_, zUpper_);
      glTexCoord2f(0.0, 0.0); glVertex3f(xLower_, yUpper_, zLower_);
      glTexCoord2f(1.0, 0.0); glVertex3f(xUpper_, yUpper_, zLower_);
    glEnd();

    // yLower
    glNormal3f(0.0, 1.0, 0.0);
    
    glBegin(GL_QUADS);
      glTexCoord2f(0.0, 1.0); glVertex3f(xLower_, yLower_, zUpper_);
      glTexCoord2f(1.0, 1.0); glVertex3f(xUpper_, yLower_, zUpper_);
      glTexCoord2f(1.0, 0.0); glVertex3f(xUpper_, yLower_, zLower_);
      glTexCoord2f(0.0, 0.0); glVertex3f(xLower_, yLower_, zLower_);
    
    // zUpper
    glNormal3f(0.0, 0.0, -1.0);
    
    glBegin(GL_QUADS);
      glTexCoord2f(0.0, 0.0); glVertex3f(xLower_, yLower_, zUpper_);
      glTexCoord2f(0.0, 1.0); glVertex3f(xLower_, yUpper_, zUpper_);
      glTexCoord2f(1.0, 1.0); glVertex3f(xUpper_, yUpper_, zUpper_);
      glTexCoord2f(1.0, 0.0); glVertex3f(xUpper_, yLower_, zUpper_);
    glEnd();
    
    // zLower
    glNormal3f(0.0, 0.0, 1.0);
    
    glBegin(GL_QUADS);
      glTexCoord2f(0.0, 0.0); glVertex3f(xLower_, yLower_, zLower_);
      glTexCoord2f(1.0, 0.0); glVertex3f(xUpper_, yLower_, zLower_);
      glTexCoord2f(1.0, 1.0); glVertex3f(xUpper_, yUpper_, zLower_);
      glTexCoord2f(0.0, 1.0); glVertex3f(xLower_, yUpper_, zLower_);
    glEnd();


    glDisable(GL_TEXTURE_2D);
    glDisable(GL_BLEND);

    glDisable(GL_LIGHTING);
    glColor3f(1.0, 1.0, 1.0);
    glutWireCube((double)edgeLength_);
    glEnable(GL_LIGHTING);
    
}

