#include <math.h>
#include <iostream>
#include "ball.h"
#include "cube.h"
#include "physicsEngine.h"





PhysicsEngine::PhysicsEngine(Cube *cube,Ball *ball) {
    ball_[0] = ball;
    numberOfBalls = 1;
    cube_ = cube;
}

PhysicsEngine::~PhysicsEngine() {
}

/* add another ball. */
void PhysicsEngine::addBall(Ball *neuball) {
   isRunning_ = false;
   ball_[numberOfBalls] = neuball;
   numberOfBalls++;   
}

void PhysicsEngine::init() {
  isRunning_ = false;
}

void PhysicsEngine::start() {
  isRunning_ = true;
}

void PhysicsEngine::toggle() {
  isRunning_ = !isRunning_;
}


/* updates the objects for one time step */
void PhysicsEngine::step() {
  float dampingCoefficient = 0.02; // 0.02; // damping gamma in motion equation. TODO: get that from main
  float springDamping = 0.4; // 0.4; // spring damping coefficient. TODO: get that from main
  float timeStep = 0.1; // 0.1; // TODO: get that from main
  float friction = 0.4; // 0.4; // The friction between the ball and the cube. TODO: get that from main
  float velocityExchangeDamping = 0.95;  // When two balls collides, they change velocity with this damping rate.

  if (!isRunning_) {
    return;
  }
  int mass1, mass2;
  float springLength[3];
  float l;
  float springForce[0];
  float collisionPoint[3];
  
  /* loop over all balls */
  for (int cb = 0; cb < numberOfBalls; cb++) {
    /* loop over all mass points: */
    for (int mp = 0; mp <= ball_[cb]->numOfMassPoints; mp++) {
      // set the force to the gravity:
      ball_[cb]->massPoints_[mp].force[0] = ball_[cb]->massPoints_[mp].mass * gravity_[0]; // F=m*g
      ball_[cb]->massPoints_[mp].force[1] = ball_[cb]->massPoints_[mp].mass * gravity_[1];
      ball_[cb]->massPoints_[mp].force[2] = ball_[cb]->massPoints_[mp].mass * gravity_[2];

      // other external forces can be added here
    
    }
    /* loop over all springs (for the calculation of the internal forces): */
    for (int sp = 0; sp < ball_[cb]->numOfSprings; sp++) {
      mass1 = ball_[cb]->springs[sp].massPoint1; // first attached mass point
      mass2 = ball_[cb]->springs[sp].massPoint2; // second attached mass point
      // calculate direction:
      springLength[0] = ball_[cb]->massPoints_[mass2].position[0] - ball_[cb]->massPoints_[mass1].position[0];
      springLength[1] = ball_[cb]->massPoints_[mass2].position[1] - ball_[cb]->massPoints_[mass1].position[1];
      springLength[2] = ball_[cb]->massPoints_[mass2].position[2] - ball_[cb]->massPoints_[mass1].position[2];
      l = sqrt(springLength[0]*springLength[0] + springLength[1]*springLength[1] + springLength[2]*springLength[2]);
      springForce[0] = -ball_[cb]->springs[sp].springConstant * (l - ball_[cb]->springs[sp].initLength)*(springLength[0] / l);
      springForce[1] = -ball_[cb]->springs[sp].springConstant * (l - ball_[cb]->springs[sp].initLength)*(springLength[1] / l);
      springForce[2] = -ball_[cb]->springs[sp].springConstant * (l - ball_[cb]->springs[sp].initLength)*(springLength[2] / l);
      // add the force to the mass points:
      ball_[cb]->massPoints_[mass2].force[0] += springForce[0] /2;
      ball_[cb]->massPoints_[mass2].force[1] += springForce[1] /2;
      ball_[cb]->massPoints_[mass2].force[2] += springForce[2] /2;
      ball_[cb]->massPoints_[mass1].force[0] -= springForce[0] /2;
      ball_[cb]->massPoints_[mass1].force[1] -= springForce[1] /2;
      ball_[cb]->massPoints_[mass1].force[2] -= springForce[2] /2;
  
      // Spring Damping (see slides) with the coefficient springDamping
      ball_[cb]->massPoints_[mass2].force[0] += -springDamping * ((l- ball_[cb]->springs[sp].lastLength) / timeStep) * (springLength[0] / l) / 2;
      ball_[cb]->massPoints_[mass2].force[1] += -springDamping * ((l- ball_[cb]->springs[sp].lastLength) / timeStep) * (springLength[1] / l) / 2;
      ball_[cb]->massPoints_[mass2].force[2] += -springDamping * ((l- ball_[cb]->springs[sp].lastLength) / timeStep) * (springLength[2] / l) / 2;
      ball_[cb]->massPoints_[mass1].force[0] -= -springDamping * ((l- ball_[cb]->springs[sp].lastLength) / timeStep) * (springLength[0] / l) / 2;
      ball_[cb]->massPoints_[mass1].force[1] -= -springDamping * ((l- ball_[cb]->springs[sp].lastLength) / timeStep) * (springLength[1] / l) / 2;
      ball_[cb]->massPoints_[mass1].force[2] -= -springDamping * ((l- ball_[cb]->springs[sp].lastLength) / timeStep) * (springLength[2] / l) / 2;
      ball_[cb]->springs[sp].lastLength = l;
      /* handle penalty force */
      // set the force to zero if a point hits the wall in this direction
      ball_[cb]->massPoints_[mass2].force[0] -= ball_[cb]->massPoints_[mass2].force[0] * ball_[cb]->massPoints_[mass2].penaltyForce[0];
      ball_[cb]->massPoints_[mass2].force[1] -= ball_[cb]->massPoints_[mass2].force[1] * ball_[cb]->massPoints_[mass2].penaltyForce[1];
      ball_[cb]->massPoints_[mass2].force[2] -= ball_[cb]->massPoints_[mass2].force[2] * ball_[cb]->massPoints_[mass2].penaltyForce[2];
      ball_[cb]->massPoints_[mass1].force[0] -= ball_[cb]->massPoints_[mass1].force[0] * ball_[cb]->massPoints_[mass1].penaltyForce[0];
      ball_[cb]->massPoints_[mass1].force[1] -= ball_[cb]->massPoints_[mass1].force[1] * ball_[cb]->massPoints_[mass1].penaltyForce[1];
      ball_[cb]->massPoints_[mass1].force[2] -= ball_[cb]->massPoints_[mass1].force[2] * ball_[cb]->massPoints_[mass1].penaltyForce[2];
      // and give this force to its opposite
      ball_[cb]->massPoints_[mass1].force[0] += ball_[cb]->massPoints_[mass2].force[0] * ball_[cb]->massPoints_[mass2].penaltyForce[0];
      ball_[cb]->massPoints_[mass1].force[1] += ball_[cb]->massPoints_[mass2].force[1] * ball_[cb]->massPoints_[mass2].penaltyForce[1];
      ball_[cb]->massPoints_[mass1].force[2] += ball_[cb]->massPoints_[mass2].force[2] * ball_[cb]->massPoints_[mass2].penaltyForce[2];
      ball_[cb]->massPoints_[mass2].force[0] += ball_[cb]->massPoints_[mass1].force[0] * ball_[cb]->massPoints_[mass1].penaltyForce[0];
      ball_[cb]->massPoints_[mass2].force[1] += ball_[cb]->massPoints_[mass1].force[1] * ball_[cb]->massPoints_[mass1].penaltyForce[1];
      ball_[cb]->massPoints_[mass2].force[2] += ball_[cb]->massPoints_[mass1].force[2] * ball_[cb]->massPoints_[mass1].penaltyForce[2];
    }
    /* reset size of the bounding sphere to zero */
    ball_[cb]->radiusOfBoundingSphere = 0.0;
    
    /* make Euler step */
    /* loop over all mass points: */
    for (int mp = 0; mp <= ball_[cb]->numOfMassPoints; mp++) {
      // position update (x = x + hv):
      ball_[cb]->massPoints_[mp].position[0] += timeStep * ball_[cb]->massPoints_[mp].velocity[0];
      ball_[cb]->massPoints_[mp].position[1] += timeStep * ball_[cb]->massPoints_[mp].velocity[1];
      ball_[cb]->massPoints_[mp].position[2] += timeStep * ball_[cb]->massPoints_[mp].velocity[2];
      // velocity update:
      ball_[cb]->massPoints_[mp].velocity[0] += timeStep *
         (ball_[cb]->massPoints_[mp].force[0] - dampingCoefficient * ball_[cb]->massPoints_[mp].velocity[0]) / ball_[cb]->massPoints_[mp].mass;
      ball_[cb]->massPoints_[mp].velocity[1] += timeStep *
         (ball_[cb]->massPoints_[mp].force[1] - dampingCoefficient * ball_[cb]->massPoints_[mp].velocity[1]) / ball_[cb]->massPoints_[mp].mass;
      ball_[cb]->massPoints_[mp].velocity[2] += timeStep * 
         (ball_[cb]->massPoints_[mp].force[2] - dampingCoefficient * ball_[cb]->massPoints_[mp].velocity[2]) / ball_[cb]->massPoints_[mp].mass;
      /*** collision detection between this ball and the wall ***/
      ball_[cb]->massPoints_[mp].penaltyForce[0] = 0.0;
      ball_[cb]->massPoints_[mp].penaltyForce[1] = 0.0;
      ball_[cb]->massPoints_[mp].penaltyForce[2] = 0.0;
      if (!cube_->inside(ball_[cb]->massPoints_[mp].position[0], ball_[cb]->massPoints_[mp].position[1], ball_[cb]->massPoints_[mp].position[2]
            , &collisionPoint[0], &collisionPoint[1], &collisionPoint[2])) {
        /* We have a collision with the cube! Now reflect the movement vector at the
         * wall. */
        // update point to the collision point
        ball_[cb]->massPoints_[mp].position[0] = collisionPoint[0];
        ball_[cb]->massPoints_[mp].position[1] = collisionPoint[1];
        ball_[cb]->massPoints_[mp].position[2] = collisionPoint[2];
        if ((collisionPoint[0] == cube_->xUpper_) || (collisionPoint[0] == cube_->xLower_))  {
          // set the velocity of the direction of the wall to zero:
          ball_[cb]->massPoints_[mp].velocity[0] = 0.0;
          ball_[cb]->massPoints_[mp].velocity[1] = friction * ball_[cb]->massPoints_[mp].velocity[1];
          ball_[cb]->massPoints_[mp].velocity[2] = friction * ball_[cb]->massPoints_[mp].velocity[2];
          // penalty force:
          ball_[cb]->massPoints_[mp].penaltyForce[0] = 1.0;
        } else if ((collisionPoint[1] == cube_->yUpper_) || (collisionPoint[1] == cube_->yLower_)) {
          // set the velocity of the direction of the wall to zero:
          ball_[cb]->massPoints_[mp].velocity[0] = friction * ball_[cb]->massPoints_[mp].velocity[0];
          ball_[cb]->massPoints_[mp].velocity[1] = 0.0;
          ball_[cb]->massPoints_[mp].velocity[2] = friction * ball_[cb]->massPoints_[mp].velocity[2];
          // penalty force:
          ball_[cb]->massPoints_[mp].penaltyForce[1] = 1.0;
        } else if ((collisionPoint[2] == cube_->zUpper_) || (collisionPoint[2] == cube_->zLower_)) {
          // set the velocity of the direction of the wall to zero:
          ball_[cb]->massPoints_[mp].velocity[0] = friction * ball_[cb]->massPoints_[mp].velocity[0];
          ball_[cb]->massPoints_[mp].velocity[1] = friction * ball_[cb]->massPoints_[mp].velocity[1];
          ball_[cb]->massPoints_[mp].velocity[2] = 0.0;
          // penalty force:
          ball_[cb]->massPoints_[mp].penaltyForce[2] = 1.0;
        } else {
          std::cout << "ERROR!!" << std::endl;
        }
      }
      // update bounding sphere of this ball
      if (ball_[cb]->distance(mp, ball_[cb]->numOfMassPoints) > ball_[cb]->radiusOfBoundingSphere) {
         ball_[cb]->radiusOfBoundingSphere = ball_[cb]->distance(mp, ball_[cb]->numOfMassPoints);
      }
    }
    
    /******* collision detection between balls ********/
    // Loop over other balls:
    for (int ob = 0; ob < cb; ob++) {
      // calculate distance between centers of the two balls
      float dx = ball_[cb]->massPoints_[ball_[cb]->numOfMassPoints].position[0] - ball_[ob]->massPoints_[ball_[ob]->numOfMassPoints].position[0];
      float dy = ball_[cb]->massPoints_[ball_[cb]->numOfMassPoints].position[1] - ball_[ob]->massPoints_[ball_[ob]->numOfMassPoints].position[1];
      float dz = ball_[cb]->massPoints_[ball_[cb]->numOfMassPoints].position[2] - ball_[ob]->massPoints_[ball_[ob]->numOfMassPoints].position[2];
      float distc = sqrt(dx*dx + dy*dy + dz*dz);
      if (distc <= (ball_[cb]->radiusOfBoundingSphere + ball_[ob]->radiusOfBoundingSphere)) {
        // Two bounding spheres intersects. So a collision is possible.
        // Now we have to test each pair of vertices seperately
        
        // idea: if a vertex v1 from ball 1 is closer to the center of ball 2 than a vertex v2 from ball 2
        //          AND v2 is closer to the center of ball 1 than v1, then exchange the velocities of v1 and v2
        //          (if that's not enough, then set the position of both, v1 and v2, to the point in the middle of them.
        //           but this may introduce errors ...)
        // loop over all mass points on the surface of ball 1
        for (int mp1 = 0; mp1 < ball_[cb]->numOfMassPoints; mp1++) {
          // loop over all mass points on the surface of ball 2
          for (int mp2 = 0; mp2 < ball_[ob]->numOfMassPoints; mp2++) {
            // distance from mp1 to center of ball 2:
            float dmp1x = ball_[cb]->massPoints_[mp1].position[0] - ball_[ob]->massPoints_[ball_[ob]->numOfMassPoints].position[0];
            float dmp1y = ball_[cb]->massPoints_[mp1].position[1] - ball_[ob]->massPoints_[ball_[ob]->numOfMassPoints].position[1];
            float dmp1z = ball_[cb]->massPoints_[mp1].position[2] - ball_[ob]->massPoints_[ball_[ob]->numOfMassPoints].position[2];
            float distmp1 = sqrt(dmp1x*dmp1x + dmp1y*dmp1y + dmp1z*dmp1z);
            if (distmp1 <= ball_[ob]->distance(ball_[ob]->numOfMassPoints, mp2)) {
              // distance from mp2 to center of ball 1:
              float dmp2x = ball_[ob]->massPoints_[mp2].position[0] - ball_[cb]->massPoints_[ball_[cb]->numOfMassPoints].position[0];
              float dmp2y = ball_[ob]->massPoints_[mp2].position[1] - ball_[cb]->massPoints_[ball_[cb]->numOfMassPoints].position[1];
              float dmp2z = ball_[ob]->massPoints_[mp2].position[2] - ball_[cb]->massPoints_[ball_[cb]->numOfMassPoints].position[2];
              float distmp2 = sqrt(dmp2x*dmp2x + dmp2y*dmp2y + dmp2z*dmp2z);
              if (distmp2 <= ball_[cb]->distance(ball_[cb]->numOfMassPoints, mp1)) {
                // collision!
                // now exchange the velocities of mp1(ball 1) and mp2(ball 2) if they are not already exchanged
                // buffer for exchange:
                float buffx = ball_[ob]->massPoints_[mp2].velocity[0];
                float buffy = ball_[ob]->massPoints_[mp2].velocity[1];
                float buffz = ball_[ob]->massPoints_[mp2].velocity[2];
                // test if the velocity of mp2 already points away from ball 1
                float scalarp = ball_[ob]->massPoints_[mp2].velocity[0] * dx + ball_[ob]->massPoints_[mp2].velocity[1] * dy + ball_[ob]->massPoints_[mp2].velocity[2] * dz;
                if (scalarp >= 0) {
                  ball_[ob]->massPoints_[mp2].velocity[0] = ball_[cb]->massPoints_[mp1].velocity[0] * velocityExchangeDamping;
                  ball_[ob]->massPoints_[mp2].velocity[1] = ball_[cb]->massPoints_[mp1].velocity[1] * velocityExchangeDamping;
                  ball_[ob]->massPoints_[mp2].velocity[2] = ball_[cb]->massPoints_[mp1].velocity[2] * velocityExchangeDamping;
                }

                // test if the velocity of mp1 already points away from ball 2
                scalarp = ball_[cb]->massPoints_[mp1].velocity[0] * dx + ball_[cb]->massPoints_[mp1].velocity[1] * dy + ball_[cb]->massPoints_[mp1].velocity[2] * dz;
                if (scalarp <= 0) {
                  ball_[cb]->massPoints_[mp1].velocity[0] = buffx * velocityExchangeDamping;
                  ball_[cb]->massPoints_[mp1].velocity[1] = buffy * velocityExchangeDamping;
                  ball_[cb]->massPoints_[mp1].velocity[2] = buffz * velocityExchangeDamping;
                }
              }
            }
          }
        }
      }
    }
  } // end loop over all balls
}


void PhysicsEngine::stop() {
  isRunning_ = false;
}

void PhysicsEngine::setGravity(float gravityX, float gravityY, float gravityZ){
  gravity_[0] = gravityX;
  gravity_[1] = gravityY;
  gravity_[2] = gravityZ;
}


